
#ifndef NX_VECTOR_H
#define NX_VECTOR_H

#include <math.h>

namespace nx {

/**
 * Basic class for vectors
  */
class nxVector {
public:
	/**
	 *  Anything that has an x and y field is a Vector
	 */
	template<typename T> nxVector(const T& p) :
		x(p.x), y(p.y) {
	}

	nxVector() :
		x(0.0), y(0.0) {
	}
	nxVector(double x, double y) :
		x(x), y(y) {
	}
	nxVector(const nxVector& v) :
		x(v.x), y(v.y) {
	}

	inline bool operator==(const nxVector& v) const {
		return x == v.x && y == v.y;
	}

	inline bool operator!=(const nxVector& v) const {
		return x != v.x || y != v.y;
	}

	inline void operator+=(const nxVector& v) {
		x += v.x;
		y += v.y;
	}

	inline void operator*=(const double& s) {
		x *= s;
		y *= s;
	}

	inline nxVector operator+(const nxVector& v) const {
		return nxVector(x + v.x, y + v.y);
	}

	inline nxVector operator-(const nxVector& v) const {
		return nxVector(x - v.x, y - v.y);
	}

	inline nxVector operator-() const {
		return nxVector(-x, -y);
	}

	inline nxVector operator*(const double& s) const {
		return nxVector(x * s, y * s);
	}

	inline nxVector operator/(const double& s) const {
		return nxVector(x / s, y / s);
	}

	inline void normalize() {
		const double l = length();
		x /= l;
		y /= l;
	}

	inline double length() const {
		if(x*x + y*y > 0)
		return sqrt(x * x + y * y);
		return 0.0;
	}

	inline double length2() const {
		return (x * x + y * y);
	}

	inline double distance(const nxVector&v) const {
		return sqrt((v.x - x) * (v.x - x) + (v.y - y) * (v.y - y));
	}

	inline double distance2(const nxVector&v) const {
		return (v.x - x) * (v.x - x) + (v.y - y) * (v.y - y);
	}

	inline double operator*(const nxVector& v) const {
		return (x * v.x) + (y * v.y);
	}

	double dotProduct(const nxVector& v) const {
		return (x * v.x) + (y * v.y);
	}

	double crossProduct(const nxVector& v) const {
		return (x * v.y) - (y * v.x);
	}

	double cos(const nxVector& v) const {
		return (dotProduct(v) / (length() * v.length()));
	}

	double angle(const nxVector& v) const {
		return acos(cos(v));
	}

	inline nxVector rotate(const nxVector& v) const {
		return nxVector(v.x * x + v.y * y, -v.y * x + v.x * y);
	}

	inline nxVector rotateBack(const nxVector& v) const {
		return nxVector(v.x * x - v.y * y, v.y * x + v.x * y);
	}

	inline nxVector norm() const {
		const double l = length();
		return nxVector(x / l, y / l);
	}

	double x;
	double y;

};

inline nxVector operator*(const double& s, const nxVector&v) {
	return nxVector(v.x * s, v.y * s);
}

}
#endif
