/*
 * nxwindow.cpp
 *
 *  Created on: Mar 21, 2012
 *      Author: Dominik Kaaser
 */
#include "nxwindow.h"
#include "nxlayer.h"
#include "nxvector.h"
#include "gl.h"
#include "color.h"
#include <string>
#include <list>
using std::string;

namespace nx {

nxWindow* nxWindow::window = 0;

void nxWindow::__callback_draw() {
	get()->initDraw();
}
void nxWindow::__callback_key(unsigned char key, int x, int y) {
	get()->key((int) key, x, y);
}
void nxWindow::__callback_specialKey(int key, int x, int y) {
	get()->specialKey(key, x, y);
}
void nxWindow::__callback_reshape(int x, int y) {
	get()->reshape(x, y);
}
void nxWindow::__callback_click(int button, int state, int x, int y) {
	get()->click(button, state, x, y);
}
void nxWindow::__callback_mouseDown(int x, int y) {
	get()->mouseDown(x, y);
}
void nxWindow::__callback_mouseMove(int x, int y) {
	get()->mouseMove(x, y);
}

nxWindow::~nxWindow() {
	for (std::list<nxLayer*>::iterator it = layers.begin(); it != layers.end(); ++it)
		delete *it;
	layers.clear();
}

const nxVector nxWindow::getMousePosition() const {
	return mousePosition;
}

const nxVector nxWindow::unproject(int x, int y) const {
	double projectionMatrix[16];

	glGetDoublev(GL_PROJECTION_MATRIX, projectionMatrix);

	const double A = projectionMatrix[0];
	const double B = projectionMatrix[5];
	const double D = projectionMatrix[12];
	const double E = projectionMatrix[13];

	const double p1 = 2.0 * ((double) x / (double) windowWidth) - 1.0;
	const double p2 = 2.0 * ((double) y / (double) windowHeight) - 1.0;

	return nxVector((p1 - D) / A, (p2 - E) / B);

}
const double nxWindow::getPixelLength() const {
	return (right - left) / (double) windowWidth;
}
const nxVector nxWindow::getOffset() const {
	return nxVector(left, bottom);
}

nxWindow::nxWindow(const string& title, int* argc, char** argv) :
		nxLayer(title), windowWidth(800), windowHeight(600), windowLeft(0), windowTop(0), restoreWidth(800), restoreHeight(600), top(1.25), right(1.5), bottom(
				-0.25), left(-0.5), fullscreen(false), clickX(0), clickY(0), maxZoom(1e10), minZoom(1e-10) {

	glutInit(argc, argv);

	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);

	glutInitWindowSize(800, 600);
	glutInitWindowPosition(0, 0);

	glutCreateWindow(name.c_str());
	glutSetIconTitle(name.c_str());

	id = glutGetWindow();
	window = this;

	windowWidth = glutGet((GLenum) GLUT_WINDOW_WIDTH);
	windowHeight = glutGet((GLenum) GLUT_WINDOW_HEIGHT);

	glViewport(0, 0, windowWidth, windowHeight);

	colors::white.setAsBackground();

	glLineWidth(1.0);
	glPointSize(4.0);

	glutDisplayFunc(__callback_draw);
	glutKeyboardFunc(__callback_key);
	glutSpecialFunc(__callback_specialKey);
	glutReshapeFunc(__callback_reshape);
	glutMouseFunc(__callback_click);
	glutMotionFunc(__callback_mouseDown);
	glutPassiveMotionFunc(__callback_mouseMove);

#ifdef FREEGLUT
	glutSetOption(GLUT_ACTION_ON_WINDOW_CLOSE, GLUT_ACTION_GLUTMAINLOOP_RETURNS);
#endif

	setAntiAliasing(true);

}
void nxWindow::setAntiAliasing(bool enable) {
	if (enable) {
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glEnable(GL_BLEND);
		glEnable(GL_LINE_SMOOTH);
		glEnable(GL_POINT_SMOOTH);
		glEnable(GL_POLYGON_SMOOTH);
		glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
		glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);
		glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);
	} else {
		glDisable(GL_BLEND);
		glDisable(GL_LINE_SMOOTH);
		glDisable(GL_POINT_SMOOTH);
		glDisable(GL_POLYGON_SMOOTH);
	}
}
void nxWindow::show() {
	glutMainLoop();
}
void nxWindow::close() {
#ifdef FREEGLUT
	glutLeaveMainLoop();
#else
	exit(0);
#endif
}
void nxWindow::updateDisplay() {
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	double addWidth = 0.5 * ((right - left) - (top - bottom) * (double) windowWidth / (double) windowHeight);
	right -= addWidth;
	left += addWidth;
	gluOrtho2D(left, right, bottom, top);
	glViewport(0, 0, windowWidth, windowHeight);
	glutPostRedisplay();
}

void nxWindow::initDraw() {
	glClear(GL_COLOR_BUFFER_BIT);
	draw();
	glFlush();
	glutSwapBuffers();
	updateDisplay();
}

void nxWindow::toggleFullscreen() {
	fullscreen = !fullscreen;
	if (fullscreen) {
		windowLeft = glutGet(GLUT_WINDOW_X);
		windowTop = glutGet(GLUT_WINDOW_Y);
		restoreWidth = glutGet(GLUT_WINDOW_WIDTH);
		restoreHeight = glutGet(GLUT_WINDOW_HEIGHT);
		glutFullScreen();
		windowWidth = glutGet(GLUT_WINDOW_WIDTH);
		windowHeight = glutGet(GLUT_WINDOW_HEIGHT);
	} else {
		glutReshapeWindow(restoreWidth, restoreHeight);
		glutPositionWindow(windowLeft, windowTop);
		windowWidth = restoreWidth;
		windowHeight = restoreHeight;
	}
	glViewport(0, 0, windowWidth, windowHeight);
}
void nxWindow::setZoomBounds(const double max, const double min) {
	maxZoom = max;
	minZoom = min;
}
void nxWindow::zoom(const int x, const int y, const double factor) {
	if (factor > 1.0 && (right - left) < minZoom)
		return;
	if (factor < 1.0 && (right - left) > maxZoom)
		return;
	const double cX = left + (double) x / (double) windowWidth * (right - left);
	const double cY = bottom + (1.0 - (double) y / (double) windowHeight) * (top - bottom);
	left = cX - (cX - left) / factor;
	right = cX - (cX - right) / factor;
	top = cY - (cY - top) / factor;
	bottom = cY - (cY - bottom) / factor;
	updateDisplay();
}

}
